/**
 * Created by anthony on 12/09/2016.
 */
var gulp = require('gulp');
var tap = require('gulp-tap');
var jnLib = require('../../lib');
var path = require('path');
var _ = require('lodash');
var rename = require('gulp-rename');
var Stream = require('stream');
var merge = require('merge-stream');
var env = require('../../env');
var templatesPath = path.join(__dirname, './templates');
var platformName = 'android';
var chalk = require('chalk');

function template(options) {

    var stream = new Stream.Transform({objectMode: true});

    stream._transform = function (file, unused, callback) {
        var contents = String(file.contents);
        var compiled = _.template(contents);
        var result = compiled(options);

        file.contents = new Buffer(result);
        callback(null, file);
    };

    return stream;
}

gulp.task('android:copy-project', function () {
    var appName = _.snakeCase(jnLib.app.name);
    return gulp.src(templatesPath + '/project/**/*')
        .pipe(gulp.dest(path.join(env.buildPath, platformName, appName)));
});

gulp.task('android:create-activities', ['android:copy-project'], function () {
    var appName = _.snakeCase(jnLib.app.name);
    var app = jnLib.app;
    var destPath = path.join('app/src/main/java', _.chain(app.package_id).split('.').join('/').value());
    var opts = {
        package_id: app.package_id,
        import_activity_base: 'de.fluidco.jnandroidruntime.JnActivityBase',
        activity_base_class: 'JnActivityBase'
    };

    var streams = [];
    var allPages = _.without(_.keys(app.screens.default), 'main');
    _.each(allPages, function (pageName) {
        var className = _.startCase(pageName).replace(/\s/g, '');
        var pipe = gulp.src(path.join(templatesPath, './activity.txt'))
            .pipe(rename(className + '.java'))
            .pipe(template(_.extend({activity_name: className, page_name:pageName}, opts)))
            .pipe(gulp.dest(path.join(env.buildPath, platformName, appName, destPath)));

        streams.push(pipe);
    });

    return merge(streams);

});

gulp.task('android:expand-project-templates', ['android:create-activities'], function () {
    var activities = [];
    var app = jnLib.app;
    var appName = _.snakeCase(jnLib.app.name);
    var allPages = _.without(_.keys(app.screens.default), 'main');
    _.each(allPages, function (pageName) {
        var isMain = app.screens.default[pageName].is_main === true;
        activities.push({name: _.startCase(pageName).replace(/\s/g, ''), is_main: isMain});
    });

    var appEnv = {
        package_id: app.package_id,
        runtime_library: 'de.fluidco.jnandroidruntime:jn-android-runtime:0.1.2',
        app_name: app.name,
        app_definition_json: JSON.stringify(app, null, '\t'),
        activities: activities,
        sdk_dir: path.join(process.env.HOME, '/Library/Android/sdk')
    };
    var resDir = 'app/src/main/res/';
    var filesNames = [path.join(resDir, 'raw/app_definition.json'),
        path.join(resDir, '/values/strings.xml'),
        'app/src/main/AndroidManifest.xml',
        'app/build.gradle',
        'local.properties'];

    return gulp.src(filesNames, {cwd: path.join(env.buildPath, platformName, appName)})
        .pipe(template(appEnv))
        .pipe(gulp.dest(function (file) {
            return file.base;
        }))
});

gulp.task('platform:android', ['android:expand-project-templates'], function () {
    var projectPath = path.join(env.buildPath, platformName, _.snakeCase(jnLib.app.name));
    console.log(chalk.green('Android project created at', projectPath));

    if (require('yargs').alias('o', 'open').argv.o) {
        console.log(chalk.green('Launching project...'));
        require('child_process').spawn('open', ['-a', 'Android Studio', projectPath], {detached: true});
    }

});